program SaveTable;

{$APPTYPE CONSOLE}

{I SQLMemVer.Inc}


uses
  SysUtils,
  Windows,
  Classes,
  Db,
  SQLMemMain,
  SQLMemTypes,
  SQLMemConst,
  SQLMemCompression
;

const RecCount = 20000;
var t:    TSQLMemTable;
    f:    Extended;
    tm:   Cardinal;
    ms:   TSQLMemMemoryStream;

function GetLogFileName: AnsiString;
begin
 Result := ExtractFilePath(ParamStr(0)) + 'log.txt';
end;

procedure aaWriteToLog(s : AnsiString);
var
    s1, name: AnsiString;
    fh:       Integer;
begin
 fh := -1;
 name := GetLogFileName;
 s1 := s + #13#10;
 while (fh < 0) do
  try
   if (SysUtils.FileExists(name)) then
    fh := SysUtils.FileOpen(name,fmOpenReadWrite or fmShareDenyWrite)
   else
    fh := SysUtils.FileCreate(name);
  except
   fh := -1;
   sleep(0);
  end;
 try
  FileSeek(fh,0,soFromEnd);
  FileWrite(fh,s1[1],Length(s1));
 finally
  if (fh >= 0) then
   SysUtils.FileClose(fh);
 end;
end;

procedure DoCreateTable(t: TSQLMemTable);
begin
 t.FieldDefs.Clear;
 t.AdvFieldDefs.Clear;
 t.IndexDefs.Clear;
 t.AdvIndexDefs.Clear;
 t.ForeignKeyDefs.Clear;

 t.AdvFieldDefs.Add('id',aftAutoInc);
 t.AdvFieldDefs.Add('int1',aftInteger,0,True);
 t.AdvFieldDefs.Add('int2',aftInteger);
 t.AdvFieldDefs.Add('int3',aftInteger,0,True);
 t.AdvFieldDefs.Add('int4',aftInteger);
 t.AdvFieldDefs.Add('str1',aftChar,20,True);
 t.AdvFieldDefs.Add('str2',aftChar,20,True);
 t.AdvFieldDefs.Add('str3',aftChar,20);
 t.AdvFieldDefs.Add('str4',aftChar,20);
 t.AdvFieldDefs.Add('str5',aftChar,20);
 t.AdvFieldDefs.Add('dt1',aftDateTime,0,True);
 t.AdvFieldDefs.Add('dt2',aftDateTime,0);
 t.AdvFieldDefs.Add('dt3',aftDateTime,0);
 t.AdvFieldDefs.Add('m1',aftCurrency,0);
 t.AdvFieldDefs.Add('m2',aftCurrency,0);
 t.IndexDefs.Add('PK','id',[ixPrimary]);
 t.IndexDefs.Add('i1','int1',[ixUnique]);
 t.IndexDefs.Add('i2','m1',[]);
 t.IndexDefs.Add('i3','dt1',[]);
 t.IndexDefs.Add('i4','str2',[ixUnique]);
 t.CreateTable;
end;

procedure FillTable(t: TSQLMemTable);
var i,j,n: Integer;
begin
 n := t.FieldCount-1;
 for i := 1 to RecCount do
  begin
   t.Insert;
   for j := 1 to n do
    if (t.Fields[j].DataType = ftCurrency) then
     t.Fields[j].AsCurrency := Random * 100000 / (i * (Random + 0.01))
    else
    if (t.Fields[j].DataType = ftInteger) then
     t.Fields[j].AsInteger := i
    else
    if (t.Fields[j].DataType = ftDateTime) then
     t.Fields[j].AsDateTime := Now
    else
     t.Fields[j].AsString := 'Test_'+FloatToStr(Random * 100000 / (i * (Random + 0.01)));
   t.Post;
  end;
end;

begin
  {$IFDEF DEBUG_MEMCHECK}
  MemChk;
  {$ENDIF}
  SysUtils.DeleteFile(GetLogFileName);
  t := TSQLMemTable.Create(nil);
  ms := TSQLMemMemoryStream.Create();
  try
    t.TableName := 'test';
    tm := GetTickCount;
    DoCreateTable(t);
    t.Open;
    tm := GetTickCount - tm;
    f := tm / 1000.0;
    aaWriteToLog('Create and open time = '+#9#9+FormatFloat('#,##0.000',f));
    // fill table
    tm := GetTickCount;
    FillTable(t);
    tm := GetTickCount - tm;
    f := tm / 1000.0;
    aaWriteToLog('Insert time = '+#9#9+FormatFloat('#,##0.000',f));
    t.Close;
    // save table
    tm := GetTickCount;
    t.SaveTableToStream(ms);
    tm := GetTickCount - tm;
    f := tm / 1000.0;
    aaWriteToLog('Save time = '+#9#9+FormatFloat('#,##0.000',f));
    ms.Position := 0;
    ms.SaveToFile('test.dat');
    ms.Position := 0;
    // delete table
    tm := GetTickCount;
    t.DeleteTable(True);
    tm := GetTickCount - tm;
    f := tm / 1000.0;
    aaWriteToLog('Delete time = '+#9#9+FormatFloat('#,##0.000',f));
    // load table
    tm := GetTickCount;
    t.LoadTableFromStream(ms);
    tm := GetTickCount - tm;
    f := tm / 1000.0;
    aaWriteToLog('Load time = '+#9#9+FormatFloat('#,##0.000',f));
  finally
    ms.Free;
    t.Free;
  end;
end.
